package eis.utils;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;

   /**
    * 
    * @author PREJITH PULIKKOTTIL
    *
    */
public class SheetUtil {

    private static void removeRows(Sheet destSheet) {
        if (null != destSheet) {
            for (int i = destSheet.getFirstRowNum(); i <= destSheet.getLastRowNum(); i++) {
                Row row = destSheet.getRow(i);
                if (null != row) {
                    destSheet.removeRow(row);
                }
            }
        }
    }

    private static void addRows(Sheet destSheet, int totalRowCount) {
        if (null != destSheet) {
            for (int i = 0; i <= totalRowCount; i++) {
                destSheet.createRow(i);
            }
        }
    }

    private static void copyMergedRegion(Sheet srcSheet, Sheet destSheet) {
        for (int i = 0; i < srcSheet.getNumMergedRegions(); i++) {
            destSheet.addMergedRegion(srcSheet.getMergedRegion(i));
        }
    }

    private static void copyCell(Cell srcCell, Cell destCell, Map<Integer, CellStyle> styleMap) {
        if (styleMap != null) {
            int srcCellHashCode = srcCell.getCellStyle().hashCode();
            CellStyle newCellStyle = styleMap.get(srcCellHashCode);
            if (null == newCellStyle) {
                newCellStyle = destCell.getSheet().getWorkbook().createCellStyle();
                newCellStyle.setAlignment(srcCell.getCellStyle().getAlignment());
                newCellStyle.setBorderBottom(srcCell.getCellStyle().getBorderBottom());
                newCellStyle.setBorderLeft(srcCell.getCellStyle().getBorderLeft());
                newCellStyle.setBorderRight(srcCell.getCellStyle().getBorderRight());
                newCellStyle.setBorderTop(srcCell.getCellStyle().getBorderTop());
                newCellStyle.setDataFormat(srcCell.getCellStyle().getDataFormat());
                newCellStyle.setFillBackgroundColor(srcCell.getCellStyle().getFillBackgroundColor());
                newCellStyle.setFillForegroundColor(srcCell.getCellStyle().getFillForegroundColor());
                newCellStyle.setFillPattern(srcCell.getCellStyle().getFillPattern());
                newCellStyle.setVerticalAlignment(srcCell.getCellStyle().getVerticalAlignment());
                newCellStyle.setWrapText(srcCell.getCellStyle().getWrapText());
                styleMap.put(srcCellHashCode, newCellStyle);
            }
            destCell.setCellStyle(newCellStyle);
        }

        if (srcCell.getCellType() == Cell.CELL_TYPE_BLANK) {
            destCell.setCellType(Cell.CELL_TYPE_BLANK);
        } else if (srcCell.getCellType() == Cell.CELL_TYPE_STRING) {
            destCell.setCellValue(srcCell.getStringCellValue());
        } else if (srcCell.getCellType() == Cell.CELL_TYPE_NUMERIC) {
            destCell.setCellValue(srcCell.getNumericCellValue());
        } else if (srcCell.getCellType() == Cell.CELL_TYPE_BOOLEAN) {
            destCell.setCellValue(srcCell.getBooleanCellValue());
        } else if (srcCell.getCellType() == Cell.CELL_TYPE_FORMULA) {
            destCell.setCellFormula(srcCell.getCellFormula());
        } else if (srcCell.getCellType() == Cell.CELL_TYPE_ERROR) {
            destCell.setCellErrorValue(srcCell.getErrorCellValue());
        }
    }

    private static void copyRow(Row srcRow, Row destRow, Map<Integer, CellStyle> styleMap) {
        destRow.setHeight(srcRow.getHeight());
        for (int j = srcRow.getFirstCellNum(); j <= srcRow.getLastCellNum(); j++) {
            Cell srcCell = srcRow.getCell(j);
            if (srcCell != null) {
                Cell destCell = destRow.createCell(j);
                copyCell(srcCell, destCell, styleMap);
            }
        }
    }

    /**
     * 
     * Copy a sheet from one workbook to another workbook. 
     * 
     * @param srcSheet
     * @param destSheet
     */
    public static void copySheet(Sheet srcSheet, Sheet destSheet) {
        removeRows(destSheet);
        addRows(destSheet, srcSheet.getLastRowNum());
        copyMergedRegion(srcSheet, destSheet);
        Map<Integer, CellStyle> styleMap = new HashMap<Integer, CellStyle>();
        for (int i = srcSheet.getFirstRowNum(); i <= srcSheet.getLastRowNum(); i++) {
            Row srcRow = srcSheet.getRow(i);
            if (null == srcRow) {
                destSheet.createRow(i);
            } else {
                Row destRow = destSheet.createRow(i);
                copyRow(srcRow, destRow, styleMap);
            }
        }
    }

    public void test1() {
        try {
            System.out.println(" test1() : " + new Date(System.currentTimeMillis()));
            File templateFile = new File("C:/poiTest/Template_V2.xlsx");
            InputStream inputStream = new FileInputStream(templateFile);
            Workbook merWorkBook = WorkbookFactory.create(inputStream);
            inputStream.close();
            Sheet destPdrSheet = merWorkBook.getSheet("PDR");

            File pdrFile = new File("C:/poiTest/P23163.xlsx");
            InputStream pdrInputStream = new FileInputStream(pdrFile);
            Workbook pdrWorkBook = WorkbookFactory.create(pdrInputStream);
            pdrInputStream.close();
            Sheet srcPdrSheet = pdrWorkBook.getSheetAt(0);

            SheetUtil.copySheet(srcPdrSheet, destPdrSheet);

            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            merWorkBook.setForceFormulaRecalculation(true);
            merWorkBook.write(byteArrayOutputStream);

            FileOutputStream resultFile = new FileOutputStream(new File("C:/poiTest/outputXLSX1.xlsx"));
            byteArrayOutputStream.writeTo(resultFile);
            System.out.println(" test1() : " + new Date(System.currentTimeMillis()));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void test2() {
        try {
            System.out.println(" test2() : " + new Date(System.currentTimeMillis()));
            File templateFile = new File("C:/poiTest/Template_V2.xls");
            InputStream inputStream = new FileInputStream(templateFile);
            Workbook merWorkBook = WorkbookFactory.create(inputStream);
            inputStream.close();
            Sheet destPdrSheet = merWorkBook.getSheet("PDR");

            File pdrFile = new File("C:/poiTest/P23163.xls");
            InputStream pdrInputStream = new FileInputStream(pdrFile);
            Workbook pdrWorkBook = WorkbookFactory.create(pdrInputStream);
            pdrInputStream.close();
            Sheet srcPdrSheet = pdrWorkBook.getSheetAt(0);

            SheetUtil.copySheet(srcPdrSheet, destPdrSheet);

            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            merWorkBook.setForceFormulaRecalculation(true);
            merWorkBook.write(byteArrayOutputStream);

            FileOutputStream resultFile = new FileOutputStream(new File("C:/poiTest/outputXLS2.xls"));
            byteArrayOutputStream.writeTo(resultFile);
            System.out.println(" test2() : " + new Date(System.currentTimeMillis()));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void test3() {
        try {
            System.out.println(" test3() : " + new Date(System.currentTimeMillis()));
            File templateFile = new File("C:/poiTest/Template_V2.xlsx");
            InputStream inputStream = new FileInputStream(templateFile);
            Workbook merWorkBook = WorkbookFactory.create(inputStream);
            inputStream.close();
            Sheet destPdrSheet = merWorkBook.getSheet("PDR");

            // cross file type :- source file is .xls and destination file is type .xlsx
            File pdrFile = new File("C:/poiTest/P23163.xls"); 
            InputStream pdrInputStream = new FileInputStream(pdrFile);
            Workbook pdrWorkBook = WorkbookFactory.create(pdrInputStream);
            pdrInputStream.close();
            Sheet srcPdrSheet = pdrWorkBook.getSheetAt(0);

            SheetUtil.copySheet(srcPdrSheet, destPdrSheet);

            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            merWorkBook.setForceFormulaRecalculation(true);
            merWorkBook.write(byteArrayOutputStream);

            FileOutputStream resultFile = new FileOutputStream(new File("C:/poiTest/outputXLSX3.xlsx"));
            byteArrayOutputStream.writeTo(resultFile);
            System.out.println(" test3() : " + new Date(System.currentTimeMillis()));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void test4() {
        try {
            System.out.println(" test4() : " + new Date(System.currentTimeMillis()));
            File templateFile = new File("C:/poiTest/Template_V2.xls");
            InputStream inputStream = new FileInputStream(templateFile);
            Workbook merWorkBook = WorkbookFactory.create(inputStream);
            inputStream.close();
            Sheet destPdrSheet = merWorkBook.getSheet("PDR");

            // cross file type source file is .xlsx and destination file type is .xls
            File pdrFile = new File("C:/poiTest/P23163.xlsx");
            InputStream pdrInputStream = new FileInputStream(pdrFile);
            Workbook pdrWorkBook = WorkbookFactory.create(pdrInputStream);
            pdrInputStream.close();
            Sheet srcPdrSheet = pdrWorkBook.getSheetAt(0);

            SheetUtil.copySheet(srcPdrSheet, destPdrSheet);

            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            merWorkBook.setForceFormulaRecalculation(true);
            merWorkBook.write(byteArrayOutputStream);

            FileOutputStream resultFile = new FileOutputStream(new File("C:/poiTest/outputXLS4.xls"));
            byteArrayOutputStream.writeTo(resultFile);
            System.out.println(" test4() : " + new Date(System.currentTimeMillis()));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void main(String[] args) {
        SheetUtil obj = new SheetUtil();
        obj.test1();
        obj.test2();
        obj.test3();
        obj.test4();
    }
}